/****************************************************************************
* VCGLib                                                            o o     *
* Visual and Computer Graphics Library                            o     o   *
*                                                                _   O  _   *
* Copyright(C) 2004-2016                                           \/)\/    *
* Visual Computing Lab                                            /\/|      *
* ISTI - Italian National Research Council                           |      *
*                                                                    \      *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/
/*! \file trimesh_base.cpp
\ingroup code_sample

\brief the minimal example of using the lib

This file contain a minimal example of the library, showing how to load a mesh and how to compute per vertex normals on it.

*/

#include<vcg/complex/complex.h>
#include<wrap/io_trimesh/import_off.h>
#include<wrap/io_trimesh/import_obj.h>

#include<wrap/io_trimesh/export_off.h>
#include<vcg/complex/algorithms/clean.h>

using namespace vcg;

class MyVertex; class MyEdge; class MyFace;
struct MyUsedTypes : public vcg::UsedTypes<vcg::Use<MyVertex>   ::AsVertexType,
                                           vcg::Use<MyEdge>     ::AsEdgeType,
                                           vcg::Use<MyFace>     ::AsFaceType>{};

class MyVertex  : public vcg::Vertex< MyUsedTypes, vcg::vertex::Coord3f, vcg::vertex::Normal3f, vcg::vertex::BitFlags  >{};
class MyFace    : public vcg::Face<   MyUsedTypes, vcg::face::FFAdj,  vcg::face::VertexRef, vcg::face::Color4b, vcg::face::BitFlags > {};
class MyEdge    : public vcg::Edge<   MyUsedTypes> {};

class MyMesh    : public vcg::tri::TriMesh< std::vector<MyVertex>, std::vector<MyFace> , std::vector<MyEdge>  > {};

class MyVertex0  : public vcg::Vertex< MyUsedTypes, vcg::vertex::Coord3f, vcg::vertex::BitFlags  >{};
class MyVertex1  : public vcg::Vertex< MyUsedTypes, vcg::vertex::Coord3f, vcg::vertex::Normal3f, vcg::vertex::BitFlags  >{};
class MyVertex2  : public vcg::Vertex< MyUsedTypes, vcg::vertex::Coord3f, vcg::vertex::Color4b, vcg::vertex::CurvatureDirf,
                                                    vcg::vertex::Qualityf, vcg::vertex::Normal3f, vcg::vertex::BitFlags  >{};


int main( int argc, char **argv )
{
  if(argc<2)
  {
    printf("Usage trimesh_base <meshfilename.obj>\n");
    return -1;
  }
  /*!
    */
  MyMesh m;

  if(vcg::tri::io::ImporterOFF<MyMesh>::Open(m,argv[1])!=vcg::tri::io::ImporterOFF<MyMesh>::NoError)
  {
    printf("Error reading file  %s\n",argv[1]);
    exit(0);
  }

  tri::RequirePerVertexNormal(m);
  tri::UpdateNormal<MyMesh>::PerVertexNormalized(m);
  printf("Input mesh  vn:%i fn:%i\n",m.VN(),m.FN());
  // try to remove duplicated vertices
  int unifiedVertices=tri::Clean<MyMesh>::RemoveDuplicateVertex(m);
  printf("Removed %i duplicated vertices\n",unifiedVertices);

  printf( "Mesh has %i vert and %i faces\n", m.VN(), m.FN() );

  // compact all the vertices and faces
  vcg::tri::Allocator<MyMesh>::CompactEveryVector(m);
  // save the mesh
  vcg::tri::io::ExporterOFF<MyMesh>::Save(m,"output.off");

  // initialize the FF topology
  vcg::tri::UpdateTopology<MyMesh>::FaceFace(m);

  // mark all the faces as not visited
  vcg::tri::UpdateFlags<MyMesh>::FaceClearV(m);

  //use a stack to visit all the faces
  std::stack<MyFace*> s;
  MyMesh::FaceIterator startFace = m.face.begin();

  while(startFace!= m.face.end())
  {
    // start from the first face
    s.push(&*startFace);

    Color4b niceColor = Color4b(rand()%255,rand()%255,rand()%255,255);

    // visit all the faces
    while(!s.empty())
    {
      MyFace *f=s.top();
      s.pop();
      f->SetV();
      f->C()=niceColor;
        for(int i=0;i<3;++i)
        {
          MyFace *fn=f->FFp(i);
          if(!fn->IsV())
            s.push(fn);
        }
    }
    // find the next face that is not visited
    while(startFace!= m.face.end() && startFace->IsV())
    {
      startFace++;
    }
  }
  

vcg::tri::io::ExporterOFF<MyMesh>::Save(m,"output2.off", vcg::tri::io::Mask::IOM_FACECOLOR  );
  return 0;
}
