/****************************************************************************
* VCGLib                                                            o o     *
* Visual and Computer Graphics Library                            o     o   *
*                                                                _   O  _   *
* Copyright(C) 2004-2016                                           \/)\/    *
* Visual Computing Lab                                            /\/|      *
* ISTI - Italian National Research Council                           |      *
*                                                                    \      *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/
/*! \file trimesh_base.cpp
\ingroup code_sample

\brief the minimal example of using the lib

This file contain a minimal example of the library, showing how to load a mesh and how to compute per vertex normals on it.

*/

#include<vcg/complex/complex.h>
#include<wrap/io_trimesh/import_off.h>
#include<wrap/io_trimesh/import_obj.h>

#include<wrap/io_trimesh/export_off.h>
#include<vcg/complex/algorithms/clean.h>

using namespace vcg;

class MyVertex; class MyEdge; class MyFace;
struct MyUsedTypes : public vcg::UsedTypes<vcg::Use<MyVertex>   ::AsVertexType,
                                           vcg::Use<MyEdge>     ::AsEdgeType,
                                           vcg::Use<MyFace>     ::AsFaceType>{};

class MyVertex  : public vcg::Vertex< MyUsedTypes, vcg::vertex::Coord3f, vcg::vertex::VFAdj, vcg::vertex::Normal3f, vcg::vertex::Color4b, vcg::vertex::BitFlags  >{};
class MyFace    : public vcg::Face<   MyUsedTypes, vcg::face::VFAdj,  vcg::face::VertexRef, vcg::face::Color4b, vcg::face::BitFlags > {};
class MyEdge    : public vcg::Edge<   MyUsedTypes> {};

class MyMesh    : public vcg::tri::TriMesh< std::vector<MyVertex>, std::vector<MyFace> , std::vector<MyEdge>  > {};

int main( int argc, char **argv )
{
  if(argc<2)
  {
    printf("Usage trimesh_base <meshfilename.obj>\n");
    return -1;
  }
  /*!
    */
  MyMesh m;

  if(vcg::tri::io::ImporterOFF<MyMesh>::Open(m,argv[1])!=vcg::tri::io::ImporterOFF<MyMesh>::NoError)
  {
    printf("Error reading file  %s\n",argv[1]);
    exit(0);
  }

  tri::RequirePerVertexNormal(m);
  tri::UpdateNormal<MyMesh>::PerVertexNormalized(m);
  printf("Input mesh  vn:%i fn:%i\n",m.VN(),m.FN());
  
  // Find all the vertices that can be reached with a path of at most two edges
  // using VF Adiacency

  tri::UpdateTopology<MyMesh>::VertexFace(m);
  // chose a random vertex
  MyMesh::VertexPointer startVert = &*m.vert.begin();

  vcg::face::VFIterator<MyFace> vfi(startVert); //initialize the iterator tohe first face
  startVert->C()=vcg::Color4b(Color4b::Red);

  for(;!vfi.End();++vfi)
  {
    MyFace* f = vfi.F();
    int ind = vfi.I();
    
    // three different ways to find the two vertices 
    // that are not the one we are looping around

    // 1) using a for and exclude the current vertex 
    // for(int i=0;i<3;++i)
    // if(i!=ind)
    // {
    //   MyVertex *v = f->V(i);
    //   v->C()=vcg::Color4b(Color4b::Green);
    // }

    // 2) using using modulo 3 artithmetics
    // f->V((ind+1)%3)->C()=vcg::Color4b(Color4b::Green);
    // f->V((ind+2)%3)->C()=vcg::Color4b(Color4b::Green);

    // 3) using V1 and V2 shorthands for the %3 operations  
    f->V1(ind)->C()=vcg::Color4b(Color4b::Green);
    f->V2(ind)->C()=vcg::Color4b(Color4b::Green);
  }
  
vcg::tri::io::ExporterOFF<MyMesh>::Save(m,"output2.off", vcg::tri::io::Mask::IOM_VERTCOLOR); 

  return 0;
}
