/****************************************************************************
* VCGLib                                                            o o     *
* Visual and Computer Graphics Library                            o     o   *
*                                                                _   O  _   *
* Copyright(C) 2004-2016                                           \/)\/    *
* Visual Computing Lab                                            /\/|      *
* ISTI - Italian National Research Council                           |      *
*                                                                    \      *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/
/*! \file trimesh_base.cpp
\ingroup code_sample

\brief the minimal example of using the lib

This file contain a minimal example of the library, showing how to load a mesh and how to compute per vertex normals on it.

*/

#include<vcg/complex/complex.h>
#include<vcg/complex/algorithms/update/color.h>
#include<wrap/io_trimesh/import_off.h>
#include<wrap/io_trimesh/import_obj.h>

#include<wrap/io_trimesh/export_off.h>
#include<wrap/io_trimesh/export_ply.h>
#include<vcg/complex/algorithms/clean.h>

using namespace vcg;

class MyVertex; class MyEdge; class MyFace;
struct MyUsedTypes : public vcg::UsedTypes<vcg::Use<MyVertex>   ::AsVertexType,
                                           vcg::Use<MyEdge>     ::AsEdgeType,
                                           vcg::Use<MyFace>     ::AsFaceType>{};

class MyVertex  : public vcg::Vertex< MyUsedTypes, vcg::vertex::Coord3f, vcg::vertex::VFAdj, vcg::vertex::Normal3f, vcg::vertex::Color4b, vcg::vertex::Qualityf, vcg::vertex::BitFlags  >{};
class MyFace    : public vcg::Face<   MyUsedTypes, vcg::face::VFAdj,  vcg::face::VertexRef, vcg::face::Color4b, vcg::face::BitFlags > {};
class MyEdge    : public vcg::Edge<   MyUsedTypes> {};

class MyMesh    : public vcg::tri::TriMesh< std::vector<MyVertex>, std::vector<MyFace> , std::vector<MyEdge>  > {};


/// Exercise: Substiute the stack based visit with a heap based visit
// pitfall heap in c++ need some care: use a pair of vertex pointer and quality 
// to store the heap elements 
std::vector< std::pair<MyMesh::VertexPointer,float> > heap;

void TopologicalVisit(MyMesh &m, std::vector<MyMesh::VertexPointer> &sources)
{
 // init the quality of the vertices to infinity
  tri::UpdateQuality<MyMesh>::VertexConstant(m, std::numeric_limits<float>::max() );
  // use a stack of vertex pointers
  std::vector<MyMesh::VertexPointer> stack;
  for(size_t i=0;i<sources.size();++i)
  {
    stack.push_back(sources[i]);
    sources[i]->Q()=0;
  }
  
  int cnt = 0;
  while(stack.empty()==false)
  {
    MyMesh::VertexPointer v = stack.back();
    stack.pop_back();
    cnt++;
    if(cnt%100000==0)
      printf("Visited %i vertices\n",cnt);
    for(vcg::face::VFIterator<MyFace> vfi(v);!vfi.End();++vfi)
    {
      MyFace* f = vfi.F();
      int ind = vfi.I();
      
      for(int i=1;i<3;++i)
      {
        float d = Distance(v->P(),f->V((ind+i)%3)->P());
        if(f->V((ind+i)%3)->Q() > v->Q()+d)
        {
          f->V((ind+i)%3)->Q() = v->Q()+d;
          stack.push_back(f->V((ind+i)%3));
        }
      }
    }
  }
}

MyMesh::VertexPointer FindFarthestVertex(MyMesh &m)
{
  MyMesh::VertexPointer farthest = &m.vert[0];
  for(size_t i=0;i<m.vn;++i)
  {
    if(m.vert[i].Q()>farthest->Q())
      farthest = &m.vert[i];
  }
  return farthest;
}

int main( int argc, char **argv )
{
  if(argc<2)
  {
    printf("Usage trimesh_base <meshfilename.obj>\n");
    return -1;
  }
  /*!
    */
  MyMesh m;
  srand(time(NULL));

  if(vcg::tri::io::ImporterOFF<MyMesh>::Open(m,argv[1])!=vcg::tri::io::ImporterOFF<MyMesh>::NoError)
  {
    printf("Error reading file  %s\n",argv[1]);
    exit(0);
  }

  tri::RequirePerVertexNormal(m);
  tri::UpdateNormal<MyMesh>::PerVertexNormalized(m);
  printf("Input mesh  vn:%i fn:%i\n",m.VN(),m.FN());
  
  // Find all the vertices that can be reached with a path of at most two edges
  // using VF Adiacency

  tri::UpdateTopology<MyMesh>::VertexFace(m);
  
  // chose a random vertex
  std::vector<MyMesh::VertexPointer> sources;
  sources.push_back(&(m.vert[rand()%m.vn]));
  
  TopologicalVisit(m,sources);  
// now each vertex has in the quality the distance from the start vertex
// color the mesh with a color ramp according to the quality value
  sources.push_back(FindFarthestVertex(m));
  TopologicalVisit(m,sources);  

tri::UpdateColor<MyMesh>::PerVertexQualityRamp(m);
  
vcg::tri::io::ExporterPLY<MyMesh>::Save(m,"output2.ply", vcg::tri::io::Mask::IOM_VERTCOLOR|tri::io::Mask::IOM_VERTQUALITY); 

  return 0;
}
